{-# LANGUAGE OverloadedStrings #-}
module Day1.Internal (
  elfToCalories,
  Elf,
  CalorieCount,
  calories,
  elf,
  parseElfGroups,
  intOrZero,
  sortElves,
  caloriesOf
) where

import qualified Data.Text as T
import qualified Data.Text.Read as R
import Data.List.Split
import qualified Data.List as L
import Data.Either

newtype CalorieCount = CalorieCount Int
newtype Elf = Elf CalorieCount

instance Show CalorieCount where
  show (CalorieCount c) = "Calories[" ++ show c ++ "]"

instance Show Elf where
  show (Elf (CalorieCount c)) = "Elf[" ++ show c ++ "]"
  
instance Eq CalorieCount where
  (==) (CalorieCount c1) (CalorieCount c2)  = c1 == c2

instance Eq Elf where
  (==) (Elf (CalorieCount c1)) (Elf (CalorieCount c2))  = c1 == c2

instance Ord Elf where
  compare (Elf (CalorieCount c1)) (Elf (CalorieCount c2)) = compare c1 c2

elf :: Int -> Elf
elf c = Elf (CalorieCount c)

calories :: Int -> CalorieCount
calories = CalorieCount

caloriesOf :: Elf -> Int
caloriesOf (Elf (CalorieCount c)) = c

sortElves :: [Elf] -> [Elf]
sortElves = L.reverse . L.sort

elfToCalories :: T.Text -> [Elf]
elfToCalories input = map elf sums
  where
    groups = parseElfGroups input
    sums = map sum groups

parseElfGroups :: T.Text -> [[Int]]
parseElfGroups input = map (map intOrZero) groups
  where
  groups = splitWhen (== "") $ T.lines input


intOrZero:: T.Text -> Int
intOrZero text = val
  where
  (val, _) = fromRight (0, "") $ R.decimal text

